using AdventureClient;
using Aliyun.OSS;
using LobbyServer;
using Log;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Protocols;
using Protocols.gm;
using Redis;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace LobbyServer
{
    /// <summary>
    /// http监听消息处理
    /// -处理接收到的http消息
    /// </summary>
    public class HttpListenerHandler
    {
        const string ModuleName = "HttpResponseHandler";

        public static void Test()
        {

        }

        #region public


        /// <summary>
        /// 测试连接
        /// </summary>
        /// <param name="Data"></param>
        /// <returns></returns>
        public static string ReqTestConnect(string Data)
        {
            return "Hello Client!";

            //GetHeroListRsp rsp = new GetHeroListRsp();
            //rsp.errorID = GetHeroListRsp.ErrorID.Success;
            //Player player = PlayerHelp.FindPlayer(700180);
            //rsp.heroInfoLst = player.OnGetHaveHerosList();
            //return JsonConvert.SerializeObject(rsp);

        }

        /// <summary>
        /// 上传GIF
        /// </summary>
        /// <param name="Data"></param>
        /// <returns></returns>
        public static async Task<string> ReqUploadGif(string Data)
        {
            string base64 = Data.Substring(Data.IndexOf(",") + 1);      //将‘,’以前的多余字符串删除
            LogSys.Warn("[HttpResponseHandler][ReqUploadGif] Data.Length: " + Data.Length + ", base64.Length: " + base64.Length);

            // UploadGif_revc rsp = new UploadGif_revc();
            //rsp.errorID = 1;

            string key = "gif/" + DateTime.Now.ToString("yyyyMMddHHmmssffff") + ".gif";
            var ossClient = new OssClient(Env.otherConfig.ossConfig.OuterEndPoint, Env.otherConfig.ossConfig.AccessKeyId, Env.otherConfig.ossConfig.AccessKeySecret);

            Bitmap bitmap = null;//定义一个Bitmap对象，接收转换完成的图片
            string filePath = "";

            try
            {
                string dummyData = base64.Trim().Replace("%", "").Replace(",", "").Replace(" ", "+");
                if (dummyData.Length % 4 > 0)
                {
                    dummyData = dummyData.PadRight(dummyData.Length + 4 - dummyData.Length % 4, '=');
                }
                LogSys.Warn("[HttpResponseHandler][ReqUploadGif] dummyData.Length: " + dummyData.Length);
                byte[] arr = Convert.FromBase64String(dummyData);//将纯净资源Base64转换成等效的8位无符号整形数组

                MemoryStream ms = new MemoryStream(arr);//转换成无法调整大小的MemoryStream对象
                bitmap = new Bitmap(ms);//将MemoryStream对象转换成Bitmap对象

                string filename = DateTime.Now.ToString("yyyyMMddHHmmssffff");
                filePath = Path.GetTempPath() + filename + ".gif"; //获取临时文件目录路径 
                bitmap.Save(filePath, System.Drawing.Imaging.ImageFormat.Gif);
                //bitmap.Save(imagesurl2 + ".jpg", System.Drawing.Imaging.ImageFormat.Jpeg);//保存到服务器路径
                //bitmap.Save(filePath + ".bmp", System.Drawing.Imaging.ImageFormat.Bmp);
                //bitmap.Save(filePath + ".png", System.Drawing.Imaging.ImageFormat.Png);
                ms.Close();//关闭当前流，并释放所有与之关联的资源
                bitmap.Dispose();

                //string filePath = Path.GetTempFileName();// 临时文件
                //Stream stream = BytesToStream(filePath, byteArray);

                await Task.Factory.FromAsync(ossClient.BeginPutObject, ossClient.EndPutObject, Env.otherConfig.ossConfig.HeadImgBucket, key, filePath, null);
            }
            catch (Exception e)
            {
                LogSys.Error("[HttpResponseHandler][ReqUploadGif] error: " + e.Message);
                // return RspUserRequest(rsp);
            }

            string pathUrl = "http://" + Env.otherConfig.ossConfig.HeadImgBucket + "." + Env.otherConfig.ossConfig.OuterEndPoint + "/" + key;

            // rsp.path = pathUrl;
            //rsp.errorID = 0;

            // return RspUserRequest(rsp);
            return null;
        }

        private static bool serialize(string data, ref ProtoBody protoBody)
        {
            byte[] byteArray = null;
            try
            {
                byteArray = Encoding.UTF8.GetBytes(data);
            }
            catch (Exception e)
            {
                LogSys.Warn("e:" + e.Message);
                try
                {
                    byteArray = Convert.FromBase64String(data);
                }
                catch (Exception ex)
                {
                    LogSys.Warn("ex:" + ex.Message);
                    return false;
                }
            }
            if (!protoBody.SerializeFromBinary(byteArray))
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// 用户登录
        /// </summary>
        /// <returns></returns>
        public static string ReqUserLogin(string Data)
        {
            UserLoginRsp rsp = new UserLoginRsp();
            rsp.errorID = UserLoginRsp.ErrorID.Failed;

            string rspStr = string.Empty;

            ProtoBody body = new ProtoBody();
            if (!serialize(Data, ref body))
            {
                rspStr = RspUserRequest(rsp);
                return rspStr;
            }

            UserLoginReq req = (UserLoginReq)body;

            if (req.platformType == EPID.Internal)
                rspStr = InternalLogin(req.account, req.password);

            return rspStr;
        }

        public static EUpLoadJSONDataErrorCode ReqUploadFile(string uid,string filename){
            Log.LogSys.Debug($"ReqUploadFile 通知用户:{uid}");

            return EUpLoadJSONDataErrorCode.OK;
        }


        #endregion

        #region private

        /// <summary>
        /// 参数转字符串
        /// </summary>
        /// <param name="dic"></param>
        /// <returns></returns>
        private static string GetPostParamString(Dictionary<string, string> dic)
        {
            StringBuilder sb = new StringBuilder();
            string s = "";
            foreach (var item in dic)
            {
                sb.Append(item.Key + "=" + item.Value + "&");
            }
            s = sb.ToString().Substring(0, s.Length - 1);
            return s.ToString();
        }

        /// <summary>
        /// 返回用户请求
        /// </summary>
        /// <param name="rsp"></param>
        /// <returns></returns>
        private static string RspUserRequest(ProtoBody rsp)
        {
            string jsonStr = JsonConvert.SerializeObject(rsp);
            return jsonStr;
            //byte[] data;
            //ProtoExtensions.SerializeToBinary(rsp, out data);
            //return Convert.ToBase64String(data);
        }

        /// <summary>
        /// 内部登录
        /// </summary>
        /// <param name="Account"></param>
        /// <param name="PassWord"></param>
        /// <returns></returns>
        private static string InternalLogin(string account, string password)
        {
            UserLoginRsp rsp = new UserLoginRsp();
            AccountKey accKey = new AccountKey(EPID.Internal, account);

            // 检查本地内存该账号是否存在
            //var player = Env.playerMgr.FindPlayerInMemory(account);
            var player = Env.playerMgr.FindPlayerInMemory(0);
            if (null == player)
            {
                LogSys.Warn($"[{ModuleName}][InternalLogin],账号不存在 account={account}");
                rsp.errorID = UserLoginRsp.ErrorID.Failed;
                return RspUserRequest(rsp);
            }

            var roleInfo = player.LoginCheckInfo();
            if (roleInfo.AccState == EAccountState.FengHao)
            {
                LogSys.Warn($"[{ModuleName}][InternalLogin],内部登录,账号停用中 account={account}");
                rsp.errorID = UserLoginRsp.ErrorID.AccountStopUseing;
                return RspUserRequest(rsp);
            }

            if (password != roleInfo.Password)
            {
                // 密码错误
                rsp.errorID = UserLoginRsp.ErrorID.PassWord_Wrong;
                LogSys.Warn($"[{ModuleName}][InternalLogin],内部登录,密码错误 account={account}");
                return RspUserRequest(rsp);
            }

            return HandleLoginSuccess(roleInfo, rsp, player);
        }

        /// <summary>
        /// 登录成功
        /// </summary>
        /// <param name="roleInfo"></param>
        /// <param name="rsp"></param>
        /// <param name="player"></param>
        /// <returns></returns>
        private static string HandleLoginSuccess(stLoginCheckInfo roleInfo, UserLoginRsp rsp, Player player)
        {
            //player.LoginInit();// 登陆初始化

            //rsp.playerStatus = player.GetPlayerStatus();//  roleInfo.status;
            //rsp.playerData = player.GetPlayerData();//  roleInfo.playerData;;
            //rsp.gateInnerToken = randomToken(player.GetAccountID());
            //rsp.errorID = UserLoginRsp.ErrorID.Success;

            //Env.httpResponse.AddUser(player.GetAccountID(), rsp.gateInnerToken);

            return RspUserRequest(rsp);
        }


        /// <summary>
        /// 随机token值
        /// </summary>
        /// <param name="accountId"></param>
        /// <returns></returns>
        private static string randomToken(ulong accountId)
        {
            TimeSpan span = DateTime.Now - DateTime.MinValue;
            ulong TotalMilliseconds = (ulong)span.TotalMilliseconds;
            return TotalMilliseconds.ToString() + "," + accountId.ToString();
        }

        /// <summary>
        /// 错误请求处理
        /// </summary>
        /// <returns></returns>
        private static string ErrorReq()
        {
            string rsp = "error req";
            return rsp;
        }
        #endregion


    }
}
